% This file demonstrates how to call the analysis SimCO algorithms 
% and the baseline algorithms to learn the dictionaries and then use 
% the learned dictionaries for natural image denoising

% Stage 1: Learn Omega
% training data: Type II, patches extracted the noisy image
% AlgName options: ASimCO, IN-ASimCO, ASimCO-Random, ASimCO-IKSVD
% Stage 2: Image Recovery

% Parameters:
% sigma: noise level
% n: image patch size
% d: signal dimension
% l: co-sparsity
% N: number of training signals
% p:number of atoms
% lambda: Lagrangian muliplier for image recovery

% References:
% J. Dong, W. Wang, W. Dai, M. D. Plumbley, Z. Han, J. Chambers
% "Analysis SimCO Algorithms for Sparse Analysis Model Based Dictionary Learning,"
% submitted to IEEE Transactions on Signal Processing, Feburary 2015.
%
% The codes & data have been deposited to http://dx.doi.org/10.15126/surreydata.00808101.
%
% Written by Jing Dong, moderated by Wenwu Wang, version 1.0                                    
%
% If you have any questions or comments regarding this package, or if you want to 
% report any bugs or unexpected error messages, please send an e-mail to
% w.wang@surrey.ac.uk
%         
% Copyright 2015 J. Dong, W. Wang, W. Dai, M. D. Plumbley, Z. Han, and J. Chambers
% 
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose. 


AlgName = 'ASimCO';
disp(AlgName);

% load clean image
ImgName ='lena256.bmp';
A = imread(ImgName);
A = double(A);
figure; imshow(A, []);

% generate noisy image
sigma = 12.8; % noise level
AN = double(A) + sigma*randn(size(A));
figure; imshow(AN, []);
PSNRnoisy = 10*log10(255.^2/mean((AN(:)-double(A(:))).^2));

%% Learn Omega==================
n = 8; % image patch size
d = n^2; % signal dimension
l = 40; % co-sparsity
X = im2col(AN, [n, n], 'sliding');
% select trainign patches
N = 20000;
IdxN=randperm(size(X, 2));
pos=IdxN(IdxN(1:min(N,size(X, 2))));
Xntrain=X(:,pos);
% remove DC
Xntrain = double(Xntrain) - ones(d,1)*mean(Xntrain);

p = 2*d; % number of atoms
OmegaInit = normr(randn(p, d));

param.initialDictionary = OmegaInit;
param.itN =2000; 
param.cosparsity = l;
param.numOmegaUpdate = 1;

switch AlgName
    case 'ASimCO'        
        % Algorithm 1: ASimCO
        [Omega] = analysis_SimCO_normalizedRowOmega(Xntrain, param);        
    case 'IN-ASimCO'
        % Algorithm 2: IN-ASimCO
        param.mu = 0.2;
        [Omega] = In_AnalysisSimCO(Xntrain, param);
    case 'ASimCO-Random'
        % Algorithm 3: ASimCO-Random
        param.mu = 0.2;
        [Omega] = ASimCO_RandomVectorDecor(Xntrain, param);
    case 'ASimCO-IKSVD'
        % Algorithm 4: ASimCO-IKSVD
        [Omega] = ASimCO_IKSVD_Decor(Xntrain, param);        
end
% ===========================

%% Recovery Image
lambda = 0.1;
% Reshape
Y = im2col(AN, [n, n], 'sliding');
% Remove DC
Ym = mean(Y);
Y = double(Y) - ones(d,1)*Ym;

% Using a learned operator
[Xdn,err] = AnalysisL1Denoising(Y, Omega, lambda);
% Add DC
Xdn = Xdn + repmat(Ym,d,1);

% Reshape
[MM,NN]=size(AN);
Idn=col2imstep(Xdn,[MM NN],[n n]);
cnt=countcover([MM NN],[n n],[1 1]);
Idn=Idn./cnt;
Idn=max(0,min(255,Idn));
figure; imshow(Idn, []);

PSNRdn = 10*log10(255.^2/mean((Idn(:)-double(A(:))).^2));



