% Experiments with synthetic data
% AlgName options: ASimCO, IN-ASimCO, ASimCO-Random, ASimCO-IKSVD

% Parameters:
% d: signal dimension
% p: number of atoms
% l: co-sparsity
% N: number of training signals

% References:
% J. Dong, W. Wang, W. Dai, M. D. Plumbley, Z. Han, J. Chambers
% "Analysis SimCO Algorithms for Sparse Analysis Model Based Dictionary Learning,"
% submitted to IEEE Transactions on Signal Processing, Feburary 2015.
%
% The codes & data have been deposited to http://dx.doi.org/10.15126/surreydata.00808101.
%
% Written by Jing Dong, moderated by Wenwu Wang, version 1.0                                    
%
% If you have any questions or comments regarding this package, or if you want to 
% report any bugs or unexpected error messages, please send an e-mail to
% w.wang@surrey.ac.uk
%          
% Copyright 2015 J. Dong, W. Wang, W. Dai, M. D. Plumbley, Z. Han, and J. Chambers
% 
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose. 

clear
AlgName = 'ASimCO';
disp(AlgName);

%% Generate synthetic data
d=25; 
p = 50; 
l = 18;
N=50000; 

% Generate a dictionary and normalize it
Omega_true=normr(randn(p,d));

% Generate analysis signals with co-sparsity l
Y=zeros(d,N);
SuppTrue=zeros(p,N);
h=waitbar(0,'Creating signals ...');
for i=1:N
    if rem(i,10)==0
        waitbar(i/N);
    end
    pos=randperm(p);
    SuppTrue(pos(1:l),i)=1;
    Z=null(Omega_true(pos(1:l),:));
    a=randn(size(Z,2),1);
    Y(:,i)=Z*a;
end;
close(h);
Y=normc(Y);
% generate an initial dictionary
OmegaInit=normr(randn(p,d));

% parameters
Omega = OmegaInit;
IPara.gmin = 1e-5; % the minimum value of gradient
IPara.Lmin = 1e-6; %t4-t1 should be larger than Lmin
IPara.t4 = 1e-2; %the initial value of t4
IPara.rNmax = 3; %the number of iterative refinement in Part B in DictLineSearch03.m
itN=2500; % the iteration number of the outer loop
num_Omega_update = 1;  % the number of Omega-Update in each iteration

OmegaRatio = zeros(1,itN);
mean_cosparsity = zeros(1,itN);


%% main iteration of Analysis SimCO
for it=1:itN
    %% analysis sparse coding
    X_sparse=Omega*Y;
    % hard thresholding
    [X_sort, index]=sort(abs(X_sparse), 1, 'ascend');
    for i=1:1:N
        X_sparse(index(1:l ,i), i)=0;
    end
    
    %% update Omega
    for i_Omega_update = 1:num_Omega_update
        [Omega, OPara]=DictLineSearch03_analysis_rowOperation(X_sparse, Omega, Y, IPara);
    end
    
    %% decorrelation step
    switch AlgName
        case 'ASimCO'
            disp('No decorrelation step');
        case 'IN-ASimCO'
            %% use functions from SMALL project
            mu = 0.6;
            dico = dico_decorr_symetric(Omega',mu);
            Omega = dico';
        case 'ASimCO-Random'
            mu = 0.6;
            for j = 1:size(Omega,1)
                if (any(abs(Omega(1:j-1,:)*Omega(j,:)') > mu))
                    Omega(j,:)=GenerateRandomRow(Y); % vector generated randomly using the given training data
                    fprintf('Replaced row no. %d - already appeared\n', j);
                    Omega(j,:)=normr(Omega(j,:));
                end
            end
        case 'ASimCO-IKSVD'
            %% decorrelation step - same as IK-SVD
            % min \|D'*D-I \|_F^2, D = Omega';
            % steepest descent method: variable step size (as described in
            % the paper); fixed iteration numer
            % parameters for decorrelation (same as those used in the original papaer)
            alpha = 0.1;
            gamma0 = 0.1;
            sd_iter_max = 20; % fixed iteration number
            
            % minimization by steepest descent (sd) method
            D = Omega';
            for sd_iter = 1:sd_iter_max
                % update D, according to equation (6)
                gamma = gamma0*(1-alpha)/(1-alpha^sd_iter);
                D = D-gamma*D*(D'*D-eye(size(D, 2)));
                D = normc(D);
            end
            Omega = D';
    end
    
    % recovery rate
    [~,OmegaRatio(it)] = dictdist_normc(Omega',Omega_true');
    fprintf('Percent recovered rows = %.2f\n',OmegaRatio(it)*100);
    
    % mean co-sparsity
    co_supp = (abs(Omega*Y)<10e-4);
    co_sparsity = sum(co_supp);
    mean_cosparsity(it) = mean(co_sparsity);
    fprintf('average cosparsity = %.3f\n',...
        mean_cosparsity(it));
end

% plot recovery rate and mean co-sparsity
figure(1);
plot(OmegaRatio);

figure(2);
plot(mean_cosparsity);

